@@ -8,22 +8,26 @@
 
 from __future__ import annotations
 
+import sys
+import uuid
+
 import asyncio
 # import logging
 import sys
 from functools import partial
 from itertools import chain
 from typing import Callable, Optional
+from collections.abc import MutableMapping
+
 from shlex import quote
 import json
 
 from inspect import getcoroutinelocals, iscoroutine
-from clu import AMQPReply, BaseClient
+from clu import AMQPClient, AMQPReply, BaseClient
 
 from .exceptions import ProxyPartialInvokeException
 
 
-
 class Proxy():
     """A proxy client with actor commands.
 
@@ -112,11 +116,29 @@ amqpc.loop.run_until_complete(start_async_setEnabled())
     __commands = "__commands"
     __comkey = "help"
     
-    def __init__(self, client: BaseClient, actor: str):
-        self.client = client
-        self.actor = actor
+    __client = None
+
+    def __init__(self, arg1, arg2 = None, **kwargs):
+        if issubclass(type(arg1), BaseClient):
+            if arg2:
+                 self.actor = arg2
+            else:
+                raise TypeError("__init__() missing 1 required positional argument: 'name'")
+
+            self.client = arg1
+            if not Proxy.__client:
+                Proxy.__client = arg1
+
+        elif isinstance(arg1, str):
+            if not Proxy.__client:
+                Proxy.__client = AMQPClient(name=f"{sys.argv[0]}.proxy-{uuid.uuid4().hex[:8]}", **kwargs)
+            self.client = Proxy.__client
+            self.actor = arg1
+        else:
+            raise TypeError("__init__() missing required positional arguments")
+
         self.async_mode = True
-#        super().__init__(client, actor)
+
 
     def start(self):
         """Query and set actor commands."""
@@ -221,7 +243,7 @@ amqpc.loop.run_until_complete(start_async_setEnabled())
         if hasattr(ret, "status") and ret.status.did_fail:
             raise self._errorMapToException(ret.replies[-1].message['error'])
 
-        return ret.replies[-1].message
+        return ProxyDict(ret.replies[-1].message)
 
 
     @staticmethod
@@ -247,19 +269,21 @@ amqpc.loop.run_until_complete(start_async_setEnabled())
             return Exception(f'Unknown module type {mn}-{tn}:{sval}')
 
 
-def invoke(*cmds):
+def invoke(*cmds, return_exceptions:Bool=False):
     """invokes one or many commands in parallel
 
     On error it throws an exception if one of the commands fails as a dict
     with an exception and return values for every command.
     """
-    async def invoke_now(proxy, *cmds, return_exceptions=False):
+    async def invoke_now(proxy, *cmds, return_exceptions:Bool):
         if proxy and not proxy.isClientConnected():
             await proxy.client.start()
         
         ret = await asyncio.gather(*[asyncio.create_task(cmd) for cmd in cmds], 
                                    return_exceptions=True)
 
+        ret = ProxyListOfDicts([ProxyDict(r) if isinstance(r, dict) else r for r in ret])
+
         if proxy:
             await proxy.client.stop()
 
@@ -269,7 +293,6 @@ def invoke(*cmds):
                     raise ProxyPartialInvokeException(*ret)
         return ret
 
-
     first_coro = cmds[0]
     assert(iscoroutine(first_coro))
 
@@ -277,9 +300,9 @@ def invoke(*cmds):
     loop = proxy.client.loop
     
     if loop.is_running():
-        return invoke_now(None, *cmds)
+        return invoke_now(None, *cmds, return_exceptions=return_exceptions)
     else:
-        return loop.run_until_complete(invoke_now(proxy, *cmds))
+        return loop.run_until_complete(invoke_now(proxy, *cmds, return_exceptions=return_exceptions))
 
 
 def unpack(ret, *keys):
@@ -320,14 +343,50 @@ def unpack(ret, *keys):
     
     if isinstance(ret, list):
         if len(keys) > 0:
-            return [d[i] for d in ret for i in keys if i in d]
+            rkeys = [k for r in ret for k in list(r.keys())]
+            bkeys = [k for k in keys if k not in rkeys]
+            if bkeys:
+                raise KeyError(bkeys)
+
+            return [d[k] for d in ret for k in keys if k in d]
         else:
             return [val for d in ret for val in list(d.values())]
 
     if len(ret) == 1:
         return list(ret.values())[0]
-    elif len(keys) > 1:
-        return [ret[i] for i in keys]
+    elif len(keys) > 0:
+        return [ret[k] for k in keys]
     else:
         return list(ret.values())
 
+
+def flatten(d: MutableMapping, parent_key: str = '', sep: str = '.'):
+    """ flattens a dict of dicts structure """
+    def _flatten_dict_gen(d, parent_key, sep):
+        for k, v in d.items():
+            new_key = parent_key + sep + k if parent_key else k
+            if isinstance(v, MutableMapping):
+                yield from flatten(v, new_key, sep=sep).items()
+            else:
+                yield new_key, v
+
+    return ProxyDict(_flatten_dict_gen(d, parent_key, sep))
+
+
+class ProxyDict(dict):
+   """ Extra helper class for the reply dict """
+   def flatten(self):
+        return flatten(self)
+
+   def unpack(self, *keys):
+        return unpack(self, *keys)
+
+
+class ProxyListOfDicts(list):
+   """ Extra helper class for the reply list of dicts """
+   def flatten(self):
+        return ProxyListOfDicts([flatten(d) for d in self])
+
+   def unpack(self, *keys):
+        return unpack(self, *keys)
+
